<?php
/**
 * User: igorivanovic
 * Date: 7/4/13
 * Time: 11:41 PM
 * @author Igor Ivanovic
 */

class CSRFProtectorBehavior extends CActiveRecordBehavior
{
	/**
	 * Key
	 */
	const KEY   = 'CSRFProtectorBehaviorSession_key';
	/**
	 * Value
	 */
	const VALUE = 'CSRFProtectorBehaviorSession_value';
	/**
	 * Token fake name
	 */
	const TOKEN = 'csrf_token_protection';
	/**
	 * @var string
	 */
	private $_key;

	/**
	 * @var value
	 */
	private $_value;

	/**
	 * @var string
	 */
	public $tokenErrorMessage = 'Token is not valid';

	/**
	 * Return crsft field
	 * @return string
	 */
	public function createTokenField()
	{
		$owner = get_class($this->getOwner());
		$name  = $owner . '[' . $this->_key . ']';
		return CHtml::hiddenField($name, $this->_value);
	}

	/**
	 * Create token
	 * @param bool $forceRefresh
	 * @return bool
	 */
	public function createToken($forceRefresh = false)
	{

		if (isset(Yii::app()->request->cookies[self::KEY]) && !$forceRefresh) {
			return false;
		}




		$this->_key   = md5('key_' . microtime(true) . rand() . rand() . rand() . rand() . rand() . rand());
		$this->_value = md5('value_' . microtime(true) . rand() . rand() . rand() . rand() . rand() . rand());

		Yii::app()->request->cookies[self::KEY] = new CHttpCookie(self::KEY, $this->_key, array('expire'=> time() + (60*60)));
		Yii::app()->request->cookies[self::VALUE] = new CHttpCookie(self::VALUE, $this->_value, array('expire'=> time() + (60*60)));

		return true;
	}




	/**
	 * Validate
	 * @param CModelEvent $event
	 */
	public function beforeValidate($event)
	{
		$owner = get_class($this->getOwner());

		$key   = isset(Yii::app()->request->cookies[self::KEY]) ? Yii::app()->request->cookies[self::KEY]->value : null;
		$value = isset(Yii::app()->request->cookies[self::VALUE]) ? Yii::app()->request->cookies[self::VALUE]->value : null;

		if ($key === null) {
			$event->sender->addError(self::TOKEN, $this->tokenErrorMessage);
		}

		$compare = isset($_REQUEST[$owner][$key]) ? $_REQUEST[$owner][$key] : null;

		if($value !== $compare){
			$event->sender->addError(self::TOKEN, $this->tokenErrorMessage);

		}

		return parent::beforeValidate($event);
	}
}